@@ -38,13 +38,7 @@ read_file( libspectrum_byte **buffer, size_t *length, const char *filename )
   }
 
   *length = info.st_size;
-
-  *buffer = malloc( *length );
-  if( !*buffer ) {
-    fprintf( stderr, "%s: out of memory allocating %lu bytes at %s:%d\n",
-	     progname, (unsigned long)*length, __func__, __LINE__ );
-    return -ENOMEM;
-  }
+  *buffer = libspectrum_malloc( *length );
 
   bytes = read( fd, *buffer, *length );
   if( bytes == -1 ) {
@@ -75,21 +69,18 @@ load_tape( libspectrum_tape **tape, const char *filename,
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_tape_alloc( tape ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  *tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_read( *tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename ) != expected_result ) {
     fprintf( stderr, "%s: reading `%s' did not give expected result\n",
 	     progname, filename );
     libspectrum_tape_free( *tape );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   return TEST_PASS;
 }
@@ -118,21 +109,18 @@ read_snap( const char *filename, const char *filename_to_pass,
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_snap_alloc( &snap ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  snap = libspectrum_snap_alloc();
 
   if( libspectrum_snap_read( snap, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename_to_pass ) != expected_result ) {
     fprintf( stderr, "%s: reading `%s' did not give expected result\n",
 	     progname, filename );
     libspectrum_snap_free( snap );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   if( libspectrum_snap_free( snap ) ) return TEST_INCOMPLETE;
 
@@ -150,19 +138,16 @@ play_tape( const char *filename )
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_tape_alloc( &tape ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_read( tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename ) ) {
     libspectrum_tape_free( tape );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   do {
 
@@ -185,7 +170,7 @@ play_tape( const char *filename )
 static test_return_t
 test_1( void )
 {
-  return read_tape( "invalid.tzx", LIBSPECTRUM_ERROR_UNKNOWN );
+  return read_tape( STATIC_TEST_PATH( "invalid.tzx" ), LIBSPECTRUM_ERROR_UNKNOWN );
 }
 
 /* Test for bugs #1720238: TZX turbo blocks with zero pilot pulses and
@@ -196,25 +181,22 @@ test_2( void )
   libspectrum_byte *buffer = NULL;
   size_t filesize = 0;
   libspectrum_tape *tape;
-  const char *filename = "turbo-zeropilot.tzx";
+  const char *filename = STATIC_TEST_PATH( "turbo-zeropilot.tzx" );
   libspectrum_dword tstates;
   int flags;
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_tape_alloc( &tape ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_read( tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename ) ) {
     libspectrum_tape_free( tape );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   if( libspectrum_tape_get_next_edge( &tstates, &flags, tape ) ) {
     libspectrum_tape_free( tape );
@@ -248,7 +230,7 @@ test_3( void )
   libspectrum_byte *buffer = (libspectrum_byte*)1;
   size_t length = 0;
 
-  if( libspectrum_tape_alloc( &tape ) ) return TEST_INCOMPLETE;
+  tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_write( &buffer, &length, tape, LIBSPECTRUM_ID_TAPE_TAP ) ) {
     libspectrum_tape_free( tape );
@@ -271,7 +253,7 @@ test_3( void )
 static test_return_t
 test_4( void )
 {
-  const char *filename = "invalid.gz";
+  const char *filename = STATIC_TEST_PATH( "invalid.gz" );
   return read_snap( filename, filename, LIBSPECTRUM_ERROR_UNKNOWN );
 }
 
@@ -279,14 +261,14 @@ test_4( void )
 static test_return_t
 test_5( void )
 {
-  return read_snap( "invalid.gz", NULL, LIBSPECTRUM_ERROR_UNKNOWN );
+  return read_snap( STATIC_TEST_PATH( "invalid.gz" ), NULL, LIBSPECTRUM_ERROR_UNKNOWN );
 }
 
 /* Test for bug #1753938: pointer wraparound causes segfault */
 static test_return_t
 test_6( void )
 {
-  const char *filename = "invalid.szx";
+  const char *filename = STATIC_TEST_PATH( "invalid.szx" );
   return read_snap( filename, filename, LIBSPECTRUM_ERROR_CORRUPT );
 }
 
@@ -294,42 +276,42 @@ test_6( void )
 static test_return_t
 test_7( void )
 {
-  return read_tape( "invalid-gdb.tzx", LIBSPECTRUM_ERROR_CORRUPT );
+  return read_tape( STATIC_TEST_PATH( "invalid-gdb.tzx" ), LIBSPECTRUM_ERROR_CORRUPT );
 }
 
 /* Test for bug #1755372: empty DRB causes segfault */
 static test_return_t
 test_8( void )
 {
-  return read_tape( "empty-drb.tzx", LIBSPECTRUM_ERROR_NONE );
+  return read_tape( STATIC_TEST_PATH( "empty-drb.tzx" ), LIBSPECTRUM_ERROR_NONE );
 }
 
 /* Test for bug #1755539: problems with invalid archive info block */
 static test_return_t
 test_9( void )
 {
-  return read_tape( "invalid-archiveinfo.tzx", LIBSPECTRUM_ERROR_CORRUPT );
+  return read_tape( STATIC_TEST_PATH( "invalid-archiveinfo.tzx" ), LIBSPECTRUM_ERROR_CORRUPT );
 }
 
 /* Test for bug #1755545: invalid hardware info blocks can leak memory */
 static test_return_t
 test_10( void )
 {
-  return read_tape( "invalid-hardwareinfo.tzx", LIBSPECTRUM_ERROR_CORRUPT );
+  return read_tape( STATIC_TEST_PATH( "invalid-hardwareinfo.tzx" ), LIBSPECTRUM_ERROR_CORRUPT );
 }
 
 /* Test for bug #1756375: invalid Warajevo tape block offset causes segfault */
 static test_return_t
 test_11( void )
 {
-  return read_tape( "invalid-warajevo-blockoffset.tap", LIBSPECTRUM_ERROR_CORRUPT );
+  return read_tape( STATIC_TEST_PATH( "invalid-warajevo-blockoffset.tap" ), LIBSPECTRUM_ERROR_CORRUPT );
 }
 
 /* Test for bug #1757587: invalid custom info block causes memory leak */
 static test_return_t
 test_12( void )
 {
-  return read_tape( "invalid-custominfo.tzx", LIBSPECTRUM_ERROR_CORRUPT );
+  return read_tape( STATIC_TEST_PATH( "invalid-custominfo.tzx" ), LIBSPECTRUM_ERROR_CORRUPT );
 }
 
 /* Test for bug #1758860: loop end without a loop start block accesses
@@ -340,25 +322,22 @@ test_13( void )
   libspectrum_byte *buffer = NULL;
   size_t filesize = 0;
   libspectrum_tape *tape;
-  const char *filename = "loopend.tzx";
+  const char *filename = STATIC_TEST_PATH( "loopend.tzx" );
   libspectrum_dword tstates;
   int flags;
 
   if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
 
-  if( libspectrum_tape_alloc( &tape ) ) {
-    free( buffer );
-    return TEST_INCOMPLETE;
-  }
+  tape = libspectrum_tape_alloc();
 
   if( libspectrum_tape_read( tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
 			     filename ) ) {
     libspectrum_tape_free( tape );
-    free( buffer );
+    libspectrum_free( buffer );
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   if( libspectrum_tape_get_next_edge( &tstates, &flags, tape ) ) {
     libspectrum_tape_free( tape );
@@ -374,28 +353,28 @@ test_13( void )
 static test_return_t
 test_14( void )
 {
-  return play_tape( "loop.tzx" );
+  return play_tape( STATIC_TEST_PATH( "loop.tzx" ) );
 }
 
 /* Test for bug #1802607: TZX loop blocks still broken */
 static test_return_t
 test_16( void )
 {
-  return play_tape( "loop2.tzx" );
+  return play_tape( STATIC_TEST_PATH( "loop2.tzx" ) );
 }
 
 /* Test for bug #1802618: TZX jump blocks broken */
 static test_return_t
 test_17( void )
 {
-  return play_tape( "jump.tzx" );
+  return play_tape( STATIC_TEST_PATH( "jump.tzx" ) );
 }
 
 /* Test for bug #1821425: crashes writing and reading empty CSW files */
 static test_return_t
 test_18( void )
 {
-  return play_tape( "empty.csw" );
+  return play_tape( STATIC_TEST_PATH( "empty.csw" ) );
 }
 
 /* Test for bug #1828945: .tap writing code does not handle all block types */
@@ -405,7 +384,7 @@ test_19( void )
   libspectrum_byte *buffer = NULL;
   size_t length = 0;
   libspectrum_tape *tape;
-  const char *filename = "complete-tzx.tzx";
+  const char *filename = DYNAMIC_TEST_PATH( "complete-tzx.tzx" );
   test_return_t r;
 
   r = load_tape( &tape, filename, LIBSPECTRUM_ERROR_NONE );
@@ -419,7 +398,7 @@ test_19( void )
     return TEST_INCOMPLETE;
   }
 
-  free( buffer );
+  libspectrum_free( buffer );
 
   if( libspectrum_tape_free( tape ) ) return TEST_INCOMPLETE;
 
@@ -431,17 +410,221 @@ test_19( void )
 static test_return_t
 test_20( void )
 {
-  const char *filename = "sp-2000.sna.gz";
+  const char *filename = STATIC_TEST_PATH( "sp-2000.sna.gz" );
   return read_snap( filename, filename, LIBSPECTRUM_ERROR_CORRUPT );
 } 
   
 static test_return_t
 test_21( void )
 {
-  const char *filename = "sp-ffff.sna.gz";
+  const char *filename = STATIC_TEST_PATH( "sp-ffff.sna.gz" );
   return read_snap( filename, filename, LIBSPECTRUM_ERROR_CORRUPT );
 } 
 
+/* Tests for bug #2002682: .mdr code does not correctly handle write protect
+   flag */
+static test_return_t
+test_22( void )
+{
+  libspectrum_byte *buffer = NULL;
+  size_t filesize = 0;
+  libspectrum_microdrive *mdr;
+  const char *filename = STATIC_TEST_PATH( "writeprotected.mdr" );
+  test_return_t r;
+
+  if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
+
+  /* writeprotected.mdr deliberately includes an extra 0 on the end;
+     we want this in the buffer so we know what happens if we read off the
+     end of the file; however, we don't want it in the length */
+  filesize--;
+
+  mdr = libspectrum_microdrive_alloc();
+
+  if( libspectrum_microdrive_mdr_read( mdr, buffer, filesize ) ) {
+    libspectrum_microdrive_free( mdr );
+    libspectrum_free( buffer );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_free( buffer );
+
+  r = libspectrum_microdrive_write_protect( mdr ) ? TEST_PASS : TEST_FAIL;
+
+  libspectrum_microdrive_free( mdr );
+
+  return r;
+}
+
+static test_return_t
+test_23( void )
+{
+  libspectrum_byte *buffer = NULL;
+  size_t filesize = 0, length;
+  libspectrum_microdrive *mdr;
+  const char *filename = STATIC_TEST_PATH( "writeprotected.mdr" );
+  test_return_t r;
+
+  if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
+
+  /* writeprotected.mdr deliberately includes an extra 0 on the end;
+     we want this in the buffer so we know what happens if we read off the
+     end of the file; however, we don't want it in the length */
+  filesize--;
+
+  mdr = libspectrum_microdrive_alloc();
+
+  if( libspectrum_microdrive_mdr_read( mdr, buffer, filesize ) ) {
+    libspectrum_microdrive_free( mdr );
+    libspectrum_free( buffer );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_free( buffer ); buffer = NULL;
+
+  libspectrum_microdrive_mdr_write( mdr, &buffer, &length );
+
+  libspectrum_microdrive_free( mdr );
+
+  r = ( length == filesize && buffer[ length - 1 ] == 1 ) ? TEST_PASS : TEST_FAIL;
+
+  libspectrum_free( buffer );
+
+  return r;
+}
+
+static test_return_t
+test_24( void )
+{
+  const char *filename = DYNAMIC_TEST_PATH( "complete-tzx.tzx" );
+  libspectrum_byte *buffer;
+  size_t filesize;
+  libspectrum_tape *tape;
+  libspectrum_tape_iterator it;
+  libspectrum_tape_block *block;
+  libspectrum_dword expected_sizes[20] = {
+    8228020,	/* ROM */
+    3493371,	/* Turbo */
+    356310,	/* Pure tone */
+    1761,	/* Pulses */
+    1993724,	/* Pure data */
+    2163000,	/* Pause */
+    0,		/* Group start */
+    0,		/* Group end */
+    0,		/* Jump */
+    205434,	/* Pure tone */
+    0,		/* Loop start */
+    154845,	/* Pure tone */
+    0,		/* Loop end */
+    0,		/* Stop tape if in 48K mode */
+    0,		/* Comment */
+    0,		/* Message */
+    0,		/* Archive info */
+    0,		/* Hardware */
+    0,		/* Custom info */
+    771620,	/* Pure tone */
+  };
+  libspectrum_dword *next_size = &expected_sizes[ 0 ];
+  test_return_t r = TEST_PASS;
+
+  if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
+
+  tape = libspectrum_tape_alloc();
+
+  if( libspectrum_tape_read( tape, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
+			     filename ) ) {
+    libspectrum_tape_free( tape );
+    libspectrum_free( buffer );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_free( buffer );
+
+  block = libspectrum_tape_iterator_init( &it, tape );
+
+  while( block )
+  {
+    libspectrum_dword actual_size = libspectrum_tape_block_length( block ); 
+
+    if( actual_size != *next_size )
+    {
+      fprintf( stderr, "%s: block had length %lu, but expected %lu\n", progname, (unsigned long)actual_size, (unsigned long)*next_size );
+      r = TEST_FAIL;
+      break;
+    }
+
+    block = libspectrum_tape_iterator_next( &it );
+    next_size++;
+  }
+
+  if( libspectrum_tape_free( tape ) ) return TEST_INCOMPLETE;
+
+  return r;
+}
+
+static test_return_t
+test_25( void )
+{
+  const char *filename = STATIC_TEST_PATH( "empty.z80" );
+  libspectrum_byte *buffer = NULL;
+  size_t filesize = 0, length = 0;
+  libspectrum_snap *snap;
+  int flags;
+  test_return_t r = TEST_INCOMPLETE;
+
+  if( read_file( &buffer, &filesize, filename ) ) return TEST_INCOMPLETE;
+
+  snap = libspectrum_snap_alloc();
+
+  if( libspectrum_snap_read( snap, buffer, filesize, LIBSPECTRUM_ID_UNKNOWN,
+			     filename ) != LIBSPECTRUM_ERROR_NONE ) {
+    fprintf( stderr, "%s: reading `%s' failed\n", progname, filename );
+    libspectrum_snap_free( snap );
+    libspectrum_free( buffer );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_free( buffer );
+  buffer = NULL;
+
+  if( libspectrum_snap_write( &buffer, &length, &flags, snap,
+                              LIBSPECTRUM_ID_SNAPSHOT_SNA, NULL, 0 ) != 
+      LIBSPECTRUM_ERROR_NONE ) {
+    fprintf( stderr, "%s: serialising to SNA failed\n", progname );
+    libspectrum_snap_free( snap );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_snap_free( snap );
+  snap = libspectrum_snap_alloc();
+
+  if( libspectrum_snap_read( snap, buffer, length, LIBSPECTRUM_ID_SNAPSHOT_SNA,
+                             NULL ) != LIBSPECTRUM_ERROR_NONE ) {
+    fprintf( stderr, "%s: restoring from SNA failed\n", progname );
+    libspectrum_snap_free( snap );
+    libspectrum_free( buffer );
+    return TEST_INCOMPLETE;
+  }
+
+  libspectrum_free( buffer );
+
+  if( libspectrum_snap_pc( snap ) != 0x1234 ) {
+    fprintf( stderr, "%s: PC is 0x%04x, not the expected 0x1234\n", progname,
+             libspectrum_snap_pc( snap ) );
+    r = TEST_FAIL;
+  } else if( libspectrum_snap_sp( snap ) != 0x8000 ) {
+    fprintf( stderr, "%s: SP is 0x%04x, not the expected 0x8000\n", progname,
+             libspectrum_snap_sp( snap ) );
+    r = TEST_FAIL;
+  } else {
+    r = TEST_PASS;
+  }
+
+  libspectrum_snap_free( snap );
+
+  return r;
+}
+
 struct test_description {
 
   test_fn test;
@@ -472,6 +655,10 @@ static struct test_description tests[] = {
   { test_19, "Complete TZX to TAP conversion", 0 },
   { test_20, "SNA file with SP < 0x4000", 0 },
   { test_21, "SNA file with SP = 0xffff", 0 },
+  { test_22, "MDR write protection 1", 0 },
+  { test_23, "MDR write protection 2", 0 },
+  { test_24, "Complete TZX timings", 0 },
+  { test_25, "Writing SNA file", 0 },
 };
 
 static size_t test_count = sizeof( tests ) / sizeof( tests[0] );
